import os

DIRNAME = os.path.dirname(__file__)

DEBUG = True

DATABASE_ENGINE = 'sqlite3'
DATABASE_NAME = os.path.join(DIRNAME, 'treenav.db')

ROOT_URLCONF = 'treenav.tests.urls'

INSTALLED_APPS = (
    'django.contrib.contenttypes',
    'mptt',
    'treenav',
    'treenav.tests',
)



import re

from django.template import Node, NodeList, Variable, Library
from django.template import TemplateSyntaxError, VariableDoesNotExist

tag_argument = re.compile(r'(\'|")?[a-zA-Z0-9_.-]+(=|\'|")?((\'|")?[a-zA-Z0-9_.-]+(\'|")?)?') 


def parse_args_kwargs(parser, token):
    tag_name, arg_str = token.contents.split(' ', 1)
    args = []
    kwargs = {}
    
    for match in tag_argument.finditer(arg_str):
        arg = match.group()
        if '=' in arg:
            k, v = arg.split('=', 1)
            kwargs[str(k)] = v
        else:
            args.append(arg)
    
    return tag_name, args, kwargs


class CaktNode(Node):
    def __init__(self, *args, **kwargs):
        self.args = [Variable(arg) for arg in args]
        self.kwargs = dict([(k, Variable(arg)) for k, arg in kwargs.items()])
        
    def render_with_args(self, context, *args, **kwargs):
        raise Exception('render_with_args must be implemented the class that inherits CaktNode')
    
    def render(self, context):
        args = []
        for arg in self.args:
            try:
                args.append(arg.resolve(context)) 
            except VariableDoesNotExist:
                args.append(None)
        
        kwargs = {}
        for k, arg in self.kwargs.items():
            try:
                kwargs[k] = arg.resolve(context)
            except VariableDoesNotExist:
                kwargs[k] = None
        
        return self.render_with_args(context, *args, **kwargs)



from django.contrib import admin
from django import forms
from django.contrib.contenttypes import generic

from treenav import models as treenav
from treenav.forms import MenuItemForm, GenericInlineMenuItemForm


class GenericMenuItemInline(generic.GenericStackedInline):
    """
    Add this inline to your admin class to support editing related menu items
    from that model's admin page.
    """
    max_num = 1
    model = treenav.MenuItem
    form = GenericInlineMenuItemForm


class SubMenuItemInline(admin.TabularInline):
    model = treenav.MenuItem
    extra = 1
    form = MenuItemForm
    prepopulated_fields = {'slug': ('label',)}
    exclude = ('new_parent',)


class MenuItemAdmin(admin.ModelAdmin):
    list_display = (
        'menu_items',
        'slug',
        'label',
        'parent',
        'link',
        'href_link',
        'order',
        'is_enabled',
    )
    list_filter = ('parent', 'is_enabled')
    raw_id_fields = ('parent',)
    prepopulated_fields = {'slug': ('label',)}
    inlines = (SubMenuItemInline, )
    fieldsets = (
        (None, {
            'fields': ('new_parent', 'label', 'slug', 'order', 'is_enabled')
        }),
        ('URL', {
            'fields': ('link', ('content_type', 'object_id')),
            'description': "Link for this menu item, which can be one of: absolute URL, named URL, or a generic relation using get_absolute_url()"
        }),
    )
    list_editable = ('label',)
    form = MenuItemForm
    
    def menu_items(self, obj):
        if obj.level == 0:
            return obj.label
        return '&nbsp;&nbsp;&nbsp;'*obj.level + '- %s' % obj.label
    menu_items.allow_tags = True
    
    def href_link(self, obj):
        return '<a href="%s">%s</a>' % (obj.href, obj.href)
    href_link.short_description = 'HREF'
    href_link.allow_tags = True

admin.site.register(treenav.MenuItem, MenuItemAdmin)



from django.test import TestCase, Client
from django.http import HttpRequest
from django.template.context import Context
from django.template import compile_string, TemplateSyntaxError, StringOrigin

from treenav.context_processors import treenav_active
from treenav.models import MenuItem

class TreeNavTestCase(TestCase):
    def setUp(self):
        self.root = MenuItem.objects.create(
            label='Primary Navigation',
            slug='primary-nav',
            order=0,
        )
        MenuItem.objects.create(
            parent=MenuItem.objects.get(slug='primary-nav'),
            label='Our Blog',
            slug='our-blog',
            order=4,
        )
        MenuItem.objects.create(
            parent=MenuItem.objects.get(slug='primary-nav'),
            label='Home',
            slug='home',
            order=0,
        )
        MenuItem.objects.create(
            parent=MenuItem.objects.get(slug='primary-nav'),
            label='About Us',
            slug='about-us',
            order=9,
        )
    
    def test_treenav_active(self):
        request = HttpRequest()
        request.META['PATH_INFO'] = '/'
        treenav_active(request)
        
    def test_to_tree(self):
        self.root.to_tree()
        
    def test_single_level_menu(self):    
        template_str = """{% load treenav_tags %}
        {% single_level_menu "primary-nav" 0 %}
        """
        origin = StringOrigin('/')
        compiled = compile_string(template_str, origin).render(Context())
        
    def test_show_treenav(self):    
        template_str = """{% load treenav_tags %}
        {% show_treenav "primary-nav" %}
        """
        origin = StringOrigin('/')
        compiled = compile_string(template_str, origin).render(Context())



from treenav.models import MenuItem
from django.core.cache import cache


def treenav_active(request):
    menus = cache.get('menus')
    if not menus:
        menus = MenuItem.objects.filter(parent__isnull=True).all()
        cache.set('menus', menus)
    treenav_active = {}
    for menu in menus:
        root = menu.to_tree()
        active_leaf = root.set_active(request.META['PATH_INFO'])
        if active_leaf:
            treenav_active[menu.slug] = active_leaf.get_active_items()
    return {'treenav_active': treenav_active}



from django.test import TestCase, Client
from django.contrib.contenttypes.models import ContentType
from django.core.urlresolvers import reverse

from django.http import HttpRequest
from treenav.models import MenuItem
from treenav.forms import MenuItemForm

from treenav.tests.models import Team


class TreeNavTestCase(TestCase):
    urls = 'treenav.tests.urls'
    def setUp(self):
        root = MenuItem.objects.create(
            label='Primary Navigation',
            slug='primary-nav',
            order=0,
        )
        MenuItem.objects.create(
            parent=MenuItem.objects.get(slug='primary-nav'),
            label='Our Blog',
            slug='our-blog',
            order=4,
        )
        MenuItem.objects.create(
            parent=MenuItem.objects.get(slug='primary-nav'),
            label='Home',
            slug='home',
            order=0,
        )
        MenuItem.objects.create(
            parent=MenuItem.objects.get(slug='primary-nav'),
            label='About Us',
            slug='about-us',
            order=9,
        )
    
    def testTagsLevel(self):
        c = Client()
        url = reverse('treenav.tests.urls.test_view',args=('home',))
        response = c.post(url,{'pslug':'primary-nav', 'N':0} )
        self.assertEquals(response.content.count('<li'),3)
        self.assertContains(response,'depth-0')
    
    def testTagsNoPage(self):
        c = Client()
        url = reverse('treenav.tests.urls.test_view',args=('notthere',))
        response = c.post(url,{'pslug':'primary-nav', 'N':0} )
        self.assertEquals(response.content.count('<li'),3)
        self.assertContains(response,'depth-0')
    
    def testTagsLevel2(self):
        MenuItem.objects.create(
            parent=MenuItem.objects.get(slug='about-us'),
            label='Second Level',
            slug='second-level',
            order=10,
        )
        c = Client()
        url = reverse('treenav.tests.urls.test_view',args=('home',))
        response = c.post(url,{'pslug':'about-us', 'N':0} )
        self.assertEquals(response.content.count('<li'),1)
        
    def testTagsImProper(self):
        c = Client()
        url = reverse('treenav.tests.urls.test_view',args=('home',))
        response = c.post(url,{'pslug':'no-nav', 'N':10000} )
        self.assertNotContains(response,'<ul')
        
    def testHierarchy(self):
        root = MenuItem.objects.get(slug='primary-nav').to_tree()
        self.assertEqual(len(root.children), 3)
        children = ('Home', 'Our Blog', 'About Us')
        for item, expected_label in zip(root.children, children):
            self.assertEqual(item.node.label, expected_label)
    
    def testGetAbsoluteUrl(self):
        team = Team.objects.create(slug='durham-bulls')
        ct = ContentType.objects.get(app_label='tests', model='team')
        form = MenuItemForm({
            'label': 'Durham Bulls',
            'slug': 'durham-bulls',
            'order': 4,
            'content_type': ct.id,
            'object_id': team.pk,
        })
        if not form.is_valid():
            self.fail(form.errors)
        menu = form.save()
        self.assertEqual(menu.href, team.get_absolute_url())
    
    def testChangedGetAbsoluteUrl(self):
        team = Team.objects.create(slug='durham-bulls')
        ct = ContentType.objects.get(app_label='tests', model='team')
        menu = MenuItem.objects.create(
            parent=MenuItem.objects.get(slug='primary-nav'),
            label='Durham Bulls',
            slug='durham-bulls',
            order=9,
            content_type=ct,
            object_id=team.pk,
            href=team.get_absolute_url(),
        )
        # change slug and save it to fire post_save signal
        team.slug = 'wildcats'
        team.save()
        menu = MenuItem.objects.get(slug='durham-bulls')
        self.assertEqual(menu.href, team.get_absolute_url())



from django import forms
from django.core.urlresolvers import reverse, NoReverseMatch
from django.contrib.contenttypes.models import ContentType
from django.core.exceptions import ObjectDoesNotExist

from treenav.models import MenuItem
from mptt.forms import TreeNodeChoiceField


class MenuItemForm(forms.ModelForm):
    new_parent = TreeNodeChoiceField(
        queryset=MenuItem.tree.all(),
        required=False,
    )
    
    class Meta:
        model = MenuItem
    
    def __init__(self, *args, **kwargs):
        super(MenuItemForm, self).__init__(*args, **kwargs)
        if self.instance.pk:
            self.fields['new_parent'].queryset = \
                MenuItem.objects.exclude(pk=self.instance.pk)
    
    def clean_link(self):
        link = self.cleaned_data['link']
        if link and not link[0] in ('^', '/'):
            try:
                reverse(link)
            except NoReverseMatch:
                raise forms.ValidationError('URL pattern name or view not found')
        return self.cleaned_data['link']
    
    def clean(self):
        content_type = self.cleaned_data['content_type']
        object_id = self.cleaned_data['object_id']
        if (content_type and not object_id) or (not content_type and object_id):
            raise forms.ValidationError(
                "Both 'Content type' and 'Object id' must be specified to use generic relationship"
            )
        if content_type and object_id:
            try:
                obj = content_type.get_object_for_this_type(pk=object_id)
            except ObjectDoesNotExist, e:
                raise forms.ValidationError(str(e))
            try:
                obj.get_absolute_url()
            except AttributeError, e:
                raise forms.ValidationError(str(e))
        return self.cleaned_data
    
    def save(self, commit=True):
        # ## WARNING ##
        # (1) respect the caller's commit argument, so that the form will
        # have a save_m2m method when commit=False
        instance = super(MenuItemForm, self).save(commit=commit)
        # (2) if commit is False, we have to save the form anyway, because
        # the instance must be saved to call move_to
        if not commit:
            instance.save()
        # reorganize if necessary
        if self.cleaned_data['new_parent']:
            parent = self.cleaned_data['new_parent']
        else:
            parent = instance.parent
        if parent:
            try:
                node = parent.get_children().order_by('order').filter(
                    order__gte=instance.order
                ).exclude(pk=instance.id)[0]
                position = 'left'
            except IndexError:
                node = parent 
                position = 'last-child'
            instance.move_to(node, position=position)
        return instance


class GenericInlineMenuItemForm(forms.ModelForm):
    parent = TreeNodeChoiceField(
        queryset=MenuItem.tree.all()
    )
    class Meta:
        model = MenuItem
        fields = ('parent', 'label', 'slug', 'order', 'is_enabled')
